<?php
require_once ('op5/license/op5lic.php');
require_once ('op5/sysinfo.php');
class ninja_license {
	const THRESHOLD = 0.95;

	/**
	 * Holds the instance of the op5lic class for this session
	 *
	 * @var op5lic
	 */
	private $lic = false;
	/**
	 * Interface to get usage statistics
	 *
	 * @var op5sysinfo
	 */
	private $sysinfo = false;

	/**
	 * Create hooks and initialize libraries
	 */
	public function __construct() {
		Event::add('system.post_controller_constructor',
			array ($this,'check_license'));
		Event::add('ninja.show_login', array ($this,'show_login'));
		Event::add('system.pre_controller', array ($this,'cleanup_session'));

		$this->sysinfo = new op5sysinfo();
		$this->lic = new op5lic();
	}

	/**
	 * Hook to add some global notifications if license needs to
	 */
	public function check_license() {
		$controller = Event::$data;

		if ($controller instanceof Ninja_Controller) {

			foreach ($this->lic->fetch_messages('load') as $msg) {
				$controller->add_global_notification($msg);
			}

			/**
			*	Add notification for SLA-reports, Availability-reports,
			*	Alert History and Alert Summary
			*/
			foreach ($this->lic->fetch_messages('report') as $msg) {
				$controller->add_print_notification($msg);
			}

			try {
				/* Fetch if exceeding threshold, and in that case, the text */
				$monitor_usage = $this->sysinfo->get_monitor_usage();
				$exceeding = !$this->lic->query_threshold('monitor',
					$monitor_usage, self::THRESHOLD, $monitor_messages);

				/*
				 * Only show exceeding message if enable_extra_license_info, or
				 * actually exceeding the license
				 */
				if (!$this->lic->fetch_property('enable_extra_license_info')) {
					$exceeding = !$this->lic->query(
						array ('monitor' => $monitor_usage,'time_expire'=>false));
				}

				/* If exceeding, display messages */
				if ($exceeding) {
					foreach ($monitor_messages as $msg) {
						$controller->add_global_notification($msg);
					}
				}
			} catch (op5LivestatusException $e) {
				/*
				 * Don't bubble this exception further. Just add a notice bar.
				 * Bubble this further means exposing the license library code
				 */
				$controller->add_global_notification($e->getMessage());
			}
		}
	}

	/**
	 * Hook to replace login screen if system is unlicensed
	 */
	public function show_login() {
		$controller = Event::$data;
		if ($this->lic->fetch_property('unlicensed')) {
			$controller->template = $controller->add_view(
				'license_call_me_maybe');
		}
	}

	/**
	 * Destroy session if unlicensed.
	 * That means, we shouldn't be able to be logged in at all to ninja
	 */
	public function cleanup_session() {
		if ($this->lic->fetch_property('unlicensed')) {
			Session::destroy();

			Auth::disable_auth_subsystem();
		}
	}
}

new ninja_license();
