<?php
$ini_include_path = get_cfg_var('include_path');
$current_include_path = get_include_path();
set_include_path($current_include_path . PATH_SEPARATOR . $ini_include_path . PATH_SEPARATOR . '/usr/share/php');

require_once ('op5/license/op5lic.php');
require_once ('op5/sysinfo.php');

define('OP5LICENSE_ERR_LICENSE_OK', 1);
define('OP5LICENSE_ERR_EXCEEDS_LICENSE', 0);
define('OP5LICENSE_ERR_UNLICENSED_PRODUCT', 2);
define('OP5LICENSE_TYPE_REGULAR', 1);
define('OP5LICENSE_TYPE_SITE', 2);
define('OP5LICENSE_TYPE_EVALUATION', 3);
define('OP5LICENSE_TYPE_UNLICENSED', 0);
define('OP5LICENSE_TYPE_ENTERPRISE', 4);
define('OP5LICENSE_TYPE_BASIC', 5);
define('OP5LICENSE_TYPE_FREE', 6);
define('OP5LICENSE_TYPE_TRIAL', 7);
define('OP5LICENSE_CHECK_OK', 1);
define('OP5LICENSE_CHECK_ERROR', 0);
define('OP5LICENSE_CHECK_EVALUATION', 2);
define('OP5LICENSE_CHECK_EVALUATION_EXPIRED', 3);
define('OP5LICENSE_CHECK_ISSUE_DATE', 4);
define('OP5LICENSE_CHECK_EXPIRE_DATE', 5);
define('OP5LICENSE_CHECK_HOSTS_COUNT_EXCEEDS', 6);
class op5license {
	private $lic = false;
	private $sysinfo = false;
	private $usage = array ();
	private $status = OP5LICENSE_CHECK_ERROR;
	private $messages = array ();

	/**
	 * Load the license file
	 *
	 * @param $override_file string
	 *        	path to license file, optional parameter
	 */
	public function __construct($override_file = op5lic::DEFAULT_LICENSE_FILE) {
		$this->lic = new op5lic($override_file);
		$this->sysinfo = new op5sysinfo();
	}

	/**
	 * Register a product for license checking.
	 * In fact, product is interpreted as a metric, which is fetched from
	 * op5sysinfo.
	 *
	 * @param $product Name
	 *        	of product (Monitor, LogServer, Trapper, APS...)
	 */
	public function register_product($product) {
		$this->usage = $this->sysinfo->get_usage(array (strtolower($product)));
	}

	/**
	 * Check if the license is valid for the given product
	 */
	public function check() {
		if ($this->lic->query($this->usage, $messages)) {
			$this->status = OP5LICENSE_CHECK_OK;
		} else {
			$this->status = OP5LICENSE_CHECK_HOSTS_COUNT_EXCEEDS;
		}
		$this->messages = $messages;
	}

	/**
	 * Return the old-style-status-constant defining the state of the license
	 */
	public function status() {
		return $this->status;
	}

	/**
	 * Print the license bar, for logserver
	 */
	public function print_license_bar() {
		if ($this->status == OP5LICENSE_CHECK_OK)
			return;

		echo '<div id="licenseInfo" onclick="javascript:top.location.href=\'/license.php\'" style="background-color: #333333; padding: 18px 0; height: auto; font-size: 16px;">';

		foreach ($this->messages as $msg) {
			echo '<strong>' . $msg . '</strong><br />';
		}

		echo '<span style="font-family: arial; padding-left: 10px; color: #ffffff">To get access to updates via op5 and the op5 support services, you need to purchase a valid license. Click here for more information.</span></div>';
	}
}
?>
