<?php
require_once ("op5lic_drvif.php");
class op5lic_drv_op5v1 implements op5lic_drvif {
	private $metrics = array ();
	private $customer = array ();
	private $messages = array ('load' => array ());
	public function __construct($filename) {
		if (!file_exists($filename))
			throw new op5lic_skip_exception("License file not found");

		$license = @simplexml_load_file($filename);
		if (!$license)
			throw new op5lic_skip_exception(
				"Invalid license XML file (non-XML license?)");

		if (((string) $license['verifier']) != 'op5v1')
			throw new op5lic_skip_exception("Incorrect license version");

		exec("/opt/op5sys/bin/op5license-verify " . $filename, $x, $v_retval);
		if ($v_retval !== 0)
			throw new op5lic_invalid_exception("Invalid license signature");

		$allowed_types = array ('regular' => true,'site' => true,
			'evaluation' => true,'poc' => true,'enterprise' => true,
			'basic' => true,'free' => true);

		/* Initialize with default values */
		$this->metrics['time_issue'] = 0;
		$this->metrics['time_expire'] = op5lic::METRIC_INFINITY;
		$this->metrics['monitor.service'] = op5lic::METRIC_INFINITY;

		/* Extract item tags */
		foreach ($license->item as $it) {
			$name = (string) $it['name'];
			$value = (string) $it['value'];

			if (preg_match('/^(.*)LicenseType$/', $name, $matches)) {
				/* Special case if name ends with LicenseType */
				$type = strtolower($value);
				if (!isset($allowed_types[$type])) {
					throw new op5lic_invalid_exception(
						"Invalid license type '$type' for '$name'");
				}
				if ($type == 'site') {
					/* License Type: SITE */
					$this->metrics[strtolower($matches[1])] = op5lic::METRIC_INFINITY;
				}
			} else if (preg_match('/^(.*)Hosts$/', $name, $matches)) {
				/* Special case if name ends with Hosts, which means metric */
				if (!isset($this->metrics[strtolower($matches[1])]))
					$this->metrics[strtolower($matches[1])] = intval($value);
			} else {
				/* Normal fields */
				switch (strtolower($name)) {
				case 'issuedtocustomernumber':
					$this->customer['number'] = $value;
					break;
				case 'issuedtoname':
					$this->customer['name'] = $value;
					break;
				case 'issuedtoemail':
					$this->customer['email'] = $value;
					break;
				case 'issuedate':
					$this->metrics['time_issue'] = strtotime($value);
					if ($this->metrics['time_issue'] === false)
						$this->metrics['time_issue'] = 0;
					break;
				}
			}
		}

		/* Fetch extensions */
		if ($license->extension) {
			foreach ($license->extension->item as $ext) {
				$extname = (string) $ext['name'];
				$extvalue = (string) $ext['value'];
				switch (strtolower($extvalue)) {
				case 'no':
					$extvalueint = 0;
					break;
				case 'yes':
					$extvalueint = 1;
					break;
				default:
					$extvalueint = intval($extvalue);
				}
				/*
				 * Never overwrite. For example, the "LogserverHosts" has higher
				 * precedence than extension "logserver"
				 */
				if (!isset($this->metrics[strtolower($extname)]))
					$this->metrics[strtolower($extname)] = $extvalueint;
			}
		} else {
			$this->metrics['aps'] = 1;
			$this->metrics['trapper'] = 1;
			$this->metrics['peers'] = op5lic::METRIC_INFINITY;
			$this->metrics['pollers'] = op5lic::METRIC_INFINITY;
		}
	}
	public function query(array $metrics = array(), $cur_time = false) {
		$message = $this->messages;
		foreach ($metrics as $metric => $usage) {
			$license_size = 0;
			if (isset($this->metrics[$metric]))
				$license_size = $this->metrics[$metric];

			if ($license_size !== op5lic::METRIC_INFINITY) {
				if ($license_size === false)
					$license_size = 0;

				if ($usage > $license_size) {
					$valid = false;
					if ($metric == 'time_expire') {
						$message['load'][] = "Your license expired " . $this->format_time_diff(
							$usage - $license_size) . " ago";
					}
					$message['load'][] = "Your usage for $metric is $usage, your license allows $license_size";
				}
			}
		}
		return array ('metrics' => $this->metrics,'messages' => $message);
	}

	public function fetch_property($prop) {
		$props = array(
		);
		if(isset($props[$prop]))
			return $props[$prop];
		return false;
	}

	public function fetch_customer_info() {
		return $this->customer;
	}

	/**
	 * Add a message to a given context or contexts.
	 *
	 * @param $context string
	 *        	A list of contexts, or a string containing a space separated
	 *        	list of contexts
	 * @param $message string
	 *        	The message.
	 */
	public function add_message($contexts, $message) {
		if (!is_array($contexts)) {
			$contexts = explode(' ', $contexts);
		}
		foreach ($contexts as $ctx) {
			if (!isset($this->messages[$ctx])) {
				$this->messages[$ctx] = array ();
			}
			$this->messages[$ctx][] = $message;
		}
	}
}
