<?php
require_once ("op5lic_drvif.php");
class op5lic_drv_op5v2 implements op5lic_drvif {
	private $metrics = array ();
	private $customer = array ();
	private $properies = array ();
	private $messages = array ('load' => array ());
	public function __construct($filename) {
		if (!file_exists($filename))
			throw new op5lic_skip_exception("License file not found");

		$license = @simplexml_load_file($filename);
		if (!$license)
			throw new op5lic_skip_exception(
				"Invalid license XML file (non-XML license?)");

		if (((string) $license['verifier']) != 'op5v2')
			throw new op5lic_skip_exception("Incorrect license version");

		exec("/opt/op5sys/bin/op5license-verify " . $filename, $x, $v_retval);
		if ($v_retval !== 0)
			throw new op5lic_invalid_exception("Invalid license signature");

		$allowed_types = array ('regular' => true,'site' => true,
			'evaluation' => true,'poc' => true,'enterprise' => true,
			'basic' => true,'free' => true);

		/* Initialize with default values */
		$this->metrics['time_issue'] = 0;
		$this->metrics['time_expire'] = op5lic::METRIC_INFINITY;
		$this->metrics['monitor.service'] = op5lic::METRIC_INFINITY;

		$lockout_if_expired = false;
		$nagbar_if_close_to_expire = false;
		$license_type = false;

		/* Extract item tags */
		foreach ($license->item as $it) {
			$name = (string) $it['name'];
			$value = (string) $it['value'];

			/* Normal fields */
			switch (strtolower($name)) {
			case 'issuedtocustomernumber':
				$this->customer['number'] = $value;
				break;
			case 'issuedtoname':
				$this->customer['name'] = $value;
				break;
			case 'issuedate':
				$this->metrics['time_issue'] = strtotime($value);
				if ($this->metrics['time_issue'] === false)
					$this->metrics['time_issue'] = 0;
				break;
			case 'expiredate':
				$this->metrics['time_expire'] = strtotime($value);
				if ($this->metrics['time_expire'] === false) {
					$this->metrics['time_expire'] = op5lic::METRIC_INFINITY;
				} else {
					/*
					 * expiration time is from a date, not a time. Make that
					 * date inclusive
					 */
					$this->metrics['time_expire'] += 24 * 60 * 60;
				}
				break;
			}
		}

		/* Extract products */
		if ($license->products) {
			foreach ($license->products->children() as $product) {
				$prodname = (string) $product->getName();
				foreach ($product->item as $it) {
					$name = (string) $it['name'];
					$value = (string) $it['value'];
					if (preg_match('/^(.*)LicenseType$/', $name, $matches)) {
						/* Name ends with LicenseType */
						$license_type = strtolower($value);
						if (!isset($allowed_types[$license_type])) {
							throw new op5lic_invalid_exception(
								"Invalid license type '$license_type' for '$name'");
						}
						if ($license_type == 'site') {
							/* License Type: SITE */
							$this->metrics[strtolower($matches[1])] = op5lic::METRIC_INFINITY;
						}
						if ($license_type == 'free') {
							/* License Type: FREE */
							$lockout_if_expired = true;
							$nagbar_if_close_to_expire = true;
							$this->properties['enable_extra_license_info'] = true;

							$this->add_message('report',
								'This report was generated using a free license of op5 Monitor.');
						}
					} else if (preg_match('/^(.*)Hosts$/', $name, $matches)) {
						/* Name ends with Hosts, which means metric */
						if (!isset($this->metrics[strtolower($matches[1])]))
							$this->metrics[strtolower($matches[1])] = intval(
								$value);
					}
				}

				/* Fetch extensions */
				if ($product->extensions) {
					foreach ($product->extensions->item as $ext) {
						$extname = (string) $ext['name'];
						$extvalue = (string) $ext['value'];
						switch (strtolower($extvalue)) {
						case 'no':
							$extvalueint = 0;
							break;
						case 'yes':
							if ((string) $ext['Hosts']) {
								$extvalueint = (int) (string) $ext['Hosts'];
							} else {
								$extvalueint = 1;
							}
							break;
						default:
							$extvalueint = intval($extvalue);
						}
						/*
						 * Never overwrite. For example, the "LogserverHosts"
						 * has higher precedence than extension "logserver"
						 */
						if (!isset($this->metrics[strtolower($extname)]))
							$this->metrics[strtolower($extname)] = $extvalueint;
					}
				}
			}
		}
		if (time() > $this->metrics['time_expire']) {
			if ($lockout_if_expired) {
				$metrics = array ();
				if (isset($this->metrics['time_issue']))
					$metrics['time_issue'] = $this->metrics['time_issue'];
				if (isset($this->metrics['time_expire']))
					$metrics['time_expire'] = $this->metrics['time_expire'];
				throw new op5lic_invalid_exception(
					'<a href="' . op5lic::OP5_BLAFFA_URL .
						 '">Your license has expired. You need to renew your license.</a>',
						array ('messages' => $this->messages,
							'metrics' => $metrics));
			}
			$this->add_message('load',
				'<a href="' . op5lic::OP5_BLAFFA_URL .
					 '">Your license has expired. You need to renew your license.</a>');
		}
		if ($this->metrics['time_expire'] - time() <= 3600 * 24 * 15) {
			if ($nagbar_if_close_to_expire) {
				if ($license_type !== false) {
					$this->add_message('load',
						'<a href="' . op5lic::OP5_BLAFFA_URL . '">Your ' .
							 $license_type .
							 ' license is about to expire. You need to renew your license.</a>');
				}
				$this->add_message('load',
					'Your license will expire in ' . op5lic::format_time_diff(
						$this->metrics['time_expire'] - time()));
			}
		}
	}
	public function query(array $metrics = array(), $cur_time = false) {
		$message = $this->messages;
		foreach ($metrics as $metric => $usage) {
			$license_size = 0;
			if (isset($this->metrics[$metric]))
				$license_size = $this->metrics[$metric];

			if ($license_size !== op5lic::METRIC_INFINITY) {
				if ($license_size === false)
					$license_size = 0;

				if ($usage > $license_size) {
					$valid = false;
					if ($metric == 'time_expire') {
						$message['load'][] = "Your license expired " . op5lic::format_time_diff(
							$usage - $license_size) . " ago";
					}
					$message['load'][] = "Your usage for $metric is $usage, your license allows $license_size";
				}
			}
		}
		return array ('metrics' => $this->metrics,'messages' => $message);
	}
	public function fetch_property($prop) {
		if (isset($this->properties[$prop]))
			return $this->properties[$prop];
		return false;
	}
	public function fetch_customer_info() {
		return $this->customer;
	}

	/**
	 * Add a message to a given context or contexts.
	 *
	 * @param $context string
	 *        	A list of contexts, or a string containing a space separated
	 *        	list of contexts
	 * @param $message string
	 *        	The message.
	 */
	public function add_message($contexts, $message) {
		if (!is_array($contexts)) {
			$contexts = explode(' ', $contexts);
		}
		foreach ($contexts as $ctx) {
			if (!isset($this->messages[$ctx])) {
				$this->messages[$ctx] = array ();
			}
			$this->messages[$ctx][] = $message;
		}
	}
}
