<?php
require_once ("op5lic_drvif.php");
class op5lic_drv_trial implements op5lic_drvif {
	private $messages = array ();
	public function __construct($filename) {
		$start_time = $this->get_license_start();
		$end_time = $start_time + 3600 * 24 * 30; // Add 30 days in seconds

		$this->metrics['time_issue'] = $start_time;
		$this->metrics['time_expire'] = $end_time;

		$now = time();
		if ($now > $end_time) {
			throw new op5lic_invalid_exception('<a href="' . op5lic::OP5_BLAFFA_URL .
					 '">Your license has expired. You need to renew your license.</a>',
			array('metrics'=>$this->metrics));
		}

		$this->add_message('report',
			'This report was generated using a trial license of op5 Monitor.');

		if (time() - $start_time >= 3600 * 24 * 3) {
			$this->add_message('load',
				'<a href="' . op5lic::OP5_BLAFFA_URL .
					 '">Get access to tips, tutorials, support and more during your trial - click here</a>');
			$this->add_message('load',
				'Your license will expire in ' .
					 op5lic::format_time_diff($end_time - time()));
		}
	}
	public function query(array $metrics = array(), $cur_time = false) {
		$lic_metrics = $this->metrics;
		$res_metrics = array ();
		foreach ($metrics as $k => $v) {
			if (isset($lic_metrics[$k])) {
				$res_metrics[$k] = $lic_metrics[$k];
			} else {
				$res_metrics[$k] = op5lic::METRIC_INFINITY;
			}
		}
		return array ('metrics' => $res_metrics,'messages' => $this->messages);
	}

	public function fetch_property($prop) {
		$props = array(
			'enable_extra_license_info' => true,
			'unlimited' => true
		);
		if(isset($props[$prop]))
			return $props[$prop];
		return false;
	}

	public function fetch_customer_info() {
		return array();
	}

	/**
	 *
	 * @return int UNIX timestamp
	 */
	public function get_license_start() {
		exec('mon status install-time 2>/dev/null', $output, $exit_code);
		$output = intval(trim(implode("", $output)));
		if ($exit_code || !$output) {
			$output = time(); // If nothing found, assume the license starts now
		}
		return intval($output);
	}

	/**
	 * Add a message to a given context or contexts.
	 *
	 * @param $context string
	 *        	A list of contexts, or a string containing a space separated
	 *        	list of contexts
	 * @param $message string
	 *        	The message.
	 */
	public function add_message($contexts, $message) {
		if (!is_array($contexts)) {
			$contexts = explode(' ', $contexts);
		}
		foreach ($contexts as $ctx) {
			if (!isset($this->messages[$ctx])) {
				$this->messages[$ctx] = array ();
			}
			$this->messages[$ctx][] = $message;
		}
	}
}